#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       parayumpole.py
#       v0.3
#       
#       Copyright (C) 2012, 2013, 2017 Nandakumar Edamana <nandakumar@nandakumar.co.in>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; version 3.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#       
#       

import gtk,os,sys
from collections import OrderedDict
import webbrowser

ONLINE_HELP_PAGE = "https://nandakumar.co.in/apps/parayumpole/"

# TODO use gettext
def _(txt):
	return txt

class EditorPage:
	def __init__(self, parayumpoleApp):
		self.txtview_in = gtk.TextView()
		self.txtview_in.set_wrap_mode(gtk.WRAP_WORD)
		self.txtbuf_in = self.txtview_in.get_buffer()
		
		self.txtview_out = gtk.TextView()
		self.txtview_out.set_wrap_mode(gtk.WRAP_WORD)
		self.txtview_out.set_editable(False)
		self.txtbuf_out = self.txtview_out.get_buffer()
		
		self.txtbuf_in.txtbuf_out = self.txtbuf_out
		self.txtbuf_in.connect("changed", parayumpoleApp.on_txtbuf_in_changed)
		
		self.sw_in = gtk.ScrolledWindow()
		self.sw_in.set_shadow_type(1) # TODO FIXME needed?
		self.sw_in.add(self.txtview_in)
		# Scrollbar policy
		self.sw_in.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		
		self.sw_out = gtk.ScrolledWindow()
		self.sw_out.set_shadow_type(1) # TODO FIXME needed?
		self.sw_out.add(self.txtview_out)
		self.sw_out.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		
		self.vbox = gtk.VBox()
		self.vbox.add(self.sw_in)
		self.vbox.add(self.sw_out)
		
		# For easy access while handling events after embedding in notebook pages
		self.vbox.txtbuf_in = self.txtbuf_in
		self.vbox.txtbuf_out = self.txtbuf_out

class main():
	topmenus = OrderedDict([
		('file', {'label': '_File', 'children': None, 'enabled': True}),
		('edit', {'label': '_Edit', 'children': None, 'enabled': False}),
		('tabs', {'label': '_Tabs', 'children': None, 'enabled': True}),
		('help', {'label': '_Help', 'children': None, 'enabled': True})
	])
	
	def __init__(self):
		# TODO add tooltips and accelerators for menus
		self.topmenus['file']['children'] = OrderedDict([
			('save_ip',  {'label': '_Save Input', 'callback': self.on_save_input}),
			('save_op',  {'label': 'Save Output', 'callback': self.on_save_output}),
			('hsep1', {'label': '-', 'callback': None}),
			('quit',  {'label': '_Quit', 'callback': self.on_destroy}) # FIXME do not destroy directly (case: unsaved)
		]);

		self.topmenus['edit']['children'] = OrderedDict([
			('cut',   {'label': 'Cu_t', 'callback': None}), # TODO
			('copy',  {'label': '_Copy', 'callback': None}), # TODO
			('paste', {'label': '_Paste', 'callback': None}), # TODO
			('hsep2', {'label': '-', 'callback': None}), # TODO
			('selall', {'label': 'Select _All', 'callback': None}) # TODO
		]);

		self.topmenus['tabs']['children'] = OrderedDict([
			('new', {'label': '_New Tab', 'callback': self.on_mnu_tab_new})
		]);
		
		self.topmenus['help']['children'] = OrderedDict([
			('online_help', {'label': 'Online _Help', 'callback': self.on_online_help}),
			('about', {'label': '_About', 'callback': self.on_about})
		]);
		
		self.newPageNum = 1
		
		#self.share_dir = "/usr/share" # TODO FIXME
		self.create_main_window()
		self.wnd.show_all()
	
	def create_main_window(self):
		self.wnd = gtk.Window()
		self.vbox = gtk.VBox()
		self.mbr = gtk.MenuBar()
		
		for key, menuobj in self.topmenus.iteritems():
			menuobj['gtkMenuItem'] = gtk.MenuItem(menuobj['label'])
			menuobj['gtkMenuItem'].set_sensitive(menuobj['enabled'])
			if menuobj['children'] != None:
				menuobj['gtkSubMenu'] = gtk.Menu()
				menuobj['gtkMenuItem'].set_submenu(menuobj['gtkSubMenu'])
				for key, childobj in menuobj['children'].iteritems():
					if childobj['label'] != '-':
						childobj['gtkMenuItem'] = gtk.MenuItem(childobj['label'])
					else:
						childobj['gtkMenuItem'] = gtk.SeparatorMenuItem()
						
					menuobj['gtkSubMenu'].add(childobj['gtkMenuItem'])
					if childobj['callback'] != None:
						childobj['gtkMenuItem'].connect('activate', childobj['callback'])
	
		self.wnd.set_title("Parayumpole")
		self.wnd.set_icon_name("parayumpole")
		self.wnd.set_size_request(600,550)
		self.wnd.set_position(1)
					
		self.wnd.connect("destroy",self.on_destroy)

		for key, obj in self.topmenus.iteritems():
			self.mbr.add(obj['gtkMenuItem'])

		self.vbox.add(self.mbr)
		
		self.notebook = gtk.Notebook()
		self.notebook.set_action_widget(gtk.Button("+"), gtk.PACK_END) # TODO FIXME
		
		self.vbox.add(self.notebook)
		
		self.wnd.add(self.vbox)
		self.vbox.set_child_packing(self.mbr,False,False,0,0)
		
		self.on_mnu_tab_new()
	
	def on_about(self, widget = None, event = None):
		# TODO FIXME do not hardcode strings
		dia = gtk.AboutDialog()
		dia.set_title("About Parayumpole")
		dia.set_name("Parayumpole")
		dia.set_version("0.3")
		dia.set_logo_icon_name("parayumpole")
		dia.set_comments("Parayumpole is a transliteration tool, mainly supporting English to Malayalam phonetic transliteration.")
		dia.set_copyright("Copyright (C) 2012, 2013, 2017 Nandakumar Edamana <nandakumar@nandakumar.co.in>");
		dia.set_authors(["Nandakumar Edamana <nandakumar@nandakumar.co.in>"])
		
		dia.run()
		dia.destroy()
		
	def on_destroy(self, widget = None):
		sys.exit(1)
	
	def on_mnu_tab_new(self, widget = None):
		# TODO add to global array, keep track of number, set label, etc.
		page = EditorPage(self)
		self.notebook.append_page(page.vbox, gtk.Label("Tab " + str(self.newPageNum))); # TODO FIXME
		page.vbox.show_all()
		self.newPageNum += 1
	
	def on_online_help(self, widget = None):
		webbrowser.open(ONLINE_HELP_PAGE);
	
	# TODO FIXME some bugs may still exist. Check all possibilities.
	def on_save_common(self, txtbuf):
		should_i_save = True;
		
		while True:
			dlg = gtk.FileChooserDialog(_("Save Input"),
				self.wnd,
				gtk.FILE_CHOOSER_ACTION_SAVE,
				buttons = (gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
			resp = dlg.run()
			if(resp != gtk.RESPONSE_ACCEPT):
				dlg.destroy()
				break;
				
			filename = dlg.get_filename()
			if(os.path.isfile(filename)):
				dlg_confirm = gtk.MessageDialog(self.wnd,
					gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
					gtk.MESSAGE_QUESTION,
					gtk.BUTTONS_YES_NO,
					_("File already exists. Overwrite?")
				)
				resp2 = dlg_confirm.run()
				dlg_confirm.destroy()
				
				if(resp2 == gtk.RESPONSE_NO):
					should_i_save = False
				else:
					should_i_save = True
					dlg.destroy()
			else:
				should_i_save = True
				dlg.destroy()

			if should_i_save:
				f = open(filename, "w")
				text = txtbuf.get_text(txtbuf.get_start_iter(), txtbuf.get_end_iter())
				f.write(text)
				f.close()
				() # TODO
				break
		
	def on_save_input(self, widget = None):
		txtbuf = self.notebook.get_nth_page(self.notebook.get_current_page()).txtbuf_in
		self.on_save_common(txtbuf)
		
	def on_save_output(self, widget = None):
		txtbuf = self.notebook.get_nth_page(self.notebook.get_current_page()).txtbuf_out
		self.on_save_common(txtbuf)
		
	def on_txtbuf_in_changed(self, widget = None): # widget should be a txtbuf_in
		inp = widget.get_text(widget.get_start_iter(), widget.get_end_iter())
		widget.txtbuf_out.set_text(self.get_ml(inp))
	
	def get_ml(self, en):
		replace2 = ["aa","ee","oo","ai","ou"]
		replace2new = ["A","^","U","I","<"]
		replace3 = [" aa"," ee"," oo"," ou"," am"]
		replace3new = [" A"," ^"," U"," <"," `"]
		vow = ["a","A","i","^","u","U","~","e","E","I","o","O","<"]
		vowml = ["അ","ആ","ഇ","ഈ","ഉ","ഊ","ഋ","എ","ഏ","ഐ","ഒ","ഓ","ഔ"]
		cons = ["k","K","g","G","c","C","j","J","t","T","d","D","N","n","p","P","b","B","m","y","r","l","v","S","s","h","L","Z","R","f","M",".",",","'","\"",";",":","?","!","/","-","_","+","=","(",")","&","%","@","1","2","3","4","5","6","7","8","9","0"]
		consml = ["ക","ഖ","ഗ","ഘ","ച","ഛ","ജ","ഝ","ട","ഠ","ദ","ഡ","ണ","ന","പ","ഫ","ബ","ഭ","മ","യ","ര","ല","വ","ശ","സ","ഹ","ള","ഴ","റ","ഫ","മ്മ",".",",","'","\"",";",":","?","!","/","-","_","+","=","(",")","&","%","@","൧","൨","൩","൪","൫","൬","൭","൮","൯","൦"]
		symbols = ["a","A","i","^","u","U","~","e","E","I","o","O","<","`","\\"]
		symml = ["","ാ","ി","ീ","ു","ൂ","ൃ","െ","േ","ൈ","ൊ","ോ","ൗ","ം","്‍"]
		hchar = ["k","g","N","c","j","n","t","T","d","D","p","b","s","z"]
		hcharml = ["ഖ","ഘ","ങ","ച","ഝ","ഞ","ത","ഥ","ധ","ഢ","ഫ","ഭ","ഷ","ഴ"]
		buff = ""

		en = "|"+en
		ml = ""
		n = 0
		line_len = 0
		while n<len(en):
			ch = en[n]
			try:
				nxt_ch = en[n+1]
			except:
				nxt_ch = ""
			try:
				later_ch = en[n+2]
			except:
				later_ch = ""
			try:
				prv_ch = en[n-1]
			except:
				prv_ch = ""
			replace3n = 0
			for item in replace3:
				if item == ch+nxt_ch+later_ch:
					en = en[:n+1]+replace3new[replace3n]+en[n+3:]
				replace3n = replace3n+1
			replace2n = 0
			for item in replace2:
				if item == nxt_ch+later_ch:
					en = en[:n+1]+replace2new[replace2n]+en[n+3:]
				replace2n = replace2n+1
			done = False
			n = n+1

# Second scan
		n = 0
		line_len = 0
		while n<len(en):
			done = False
			ch = en[n]
			try:
				nxt_ch = en[n+1]
			except:
				nxt_ch = ""
			try:
				later_ch = en[n+2]
			except:
				later_ch = ""
			try:
				prv_ch = en[n-1]
			except:
				prv_ch = ""
				
			if ch == "\n":
				en = en[:n]+"|\n|"+en[n+1:]
				ml = ml+"\n"
				n = n+1
				
			#if done == False and ch == "n" and nxt_ch == "h" : TODO FIXME
			if done == False:
				if ch == " " or ch == "|":
					connector = ""
					if ch == " ":connector = " "
					vown = 0
					for item in vow:
						if item == nxt_ch:
							ml = ml+connector+vowml[vown]
							done = True
							n = n+2
						vown = vown+1
					if done == False:
						ml = ml+connector
						done = True
						n = n+1
			
			if done == False and ch == "n" and nxt_ch == "g":
				buff = "്"
				symn = 0
				for sym in symbols:
					if sym == later_ch:buff = symml[symn]
					symn = symn+1
				ml = ml+"ങ"+buff
				n = n+2
				done = True
			
			if done == False and ch == "n" and nxt_ch == "j":
				buff = "്"
				symn = 0
				for sym in symbols:
					if sym == later_ch:buff = symml[symn]
					symn = symn+1
				ml = ml+"ഞ"+buff
				n = n+2
				done = True
					
			if done == False and ch == "m" and nxt_ch == "p":
				buff = "്"
				symn = 0
				for sym in symbols:
					if sym == later_ch:buff = symml[symn]
					symn = symn+1
				ml = ml+"മ്പ"+buff
				n = n+2
				done = True
									
			if done == False and ch == "m":
					
				if nxt_ch<>"m" and nxt_ch<>"a" and nxt_ch<>"A" and nxt_ch<>"i" and nxt_ch<>"^" and nxt_ch<>"u" and nxt_ch<>"U" and nxt_ch<>"~" and nxt_ch<>"e" and nxt_ch<>"E" and nxt_ch<>"I" and nxt_ch<>"o" and nxt_ch<>"O" and nxt_ch<>"<":
					buff = "്"
					symn = 0
					for sym in symbols:
						if sym == prv_ch:buff = symml[symn]
						symn = symn+1
					if buff<>"്":
						ml = ml+"ം"
						n = n+1
						done = True
					else:
						done = False
				
			if done == False and nxt_ch == "h":
				hcharn = 0
				for item in hchar:
					if item == ch:
						buff = "്"
						symn = 0
						for sym in symbols:
							if sym == later_ch:buff = symml[symn]
							symn = symn+1
						ml = ml+hcharml[hcharn]+buff
						n = n+2
						done = True
					hcharn = hcharn+1
					
			if done == False and nxt_ch<>"h":	
				consn = 0
				for item in cons:
					if item == ch:
						if consn<30:
							buff = "്" # items after 37 are with 'a' in built.
						else:
							buff = ""
						symn = 0
						for sym in symbols:
							if sym == nxt_ch:buff = symml[symn]
							symn = symn+1
						ml = ml+consml[consn]+buff
					consn = consn+1
				n = n+1
				done = True
				
			if done == False:
				#ml = ml+ch
				n = n+1 # TODO FIXME remove after all conditions
			
		return ml
		
if __name__  ==  '__main__':
	main()
	gtk.main()
